import javafx.event.EventHandler;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TextField;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;

import treetools.MunicipalTree;

import java.util.Stack;

/**
 * TreeFilterEventHandler handles UI events captured via the TreeViewer
 */
public class TreeFilterEventHandler implements EventHandler<MouseEvent> {

    private ComboBox treeSelect; //select box ref
    private TextField txtSummary; //summary of tree counts
    private TreeViewer treeView; //the tree view

    /**
     * Constructor
     *
     * @param view reference to TreeView
     */
    public TreeFilterEventHandler(TreeViewer view) {
        this.treeView = view;
        this.treeSelect = view.getTreeSelect();
        this.txtSummary = view.getTxtSummary();
    }

    /**
     * Handle a mouse event (i.e. a button click)!  This routine will need to:
     * 1. Clear (or UNDO) all the circles from the existing view
     * 2. Cycle thru the list of trees and redraw circles that correspond to the tree type selected
     * 3. Remember to register any circles you draw on the "undo" stack, so they can be undone later!
     * 4. Remember also that if the user selects "ALL TREES", all the trees should be drawn
     *
     * @param event The mouse event
     */
    @Override
    public void handle(MouseEvent event) {
        
        Stack undoStack = treeView.getUndoStack();
        while (!undoStack.isEmpty()) {
            Circle circle = (Circle) undoStack.pop();
            treeView.getAnchorRoot().getChildren().remove(circle);
        }

        String targetName = ((String) treeSelect.getValue());
        boolean showAllTrees = "ALL TREES".equalsIgnoreCase(targetName);

        for (MunicipalTree tree : treeView.getTrees()) {
            float w = treeView.getWidth();
            float h = treeView.getHeight();

            double[] boundaries = treeView.getBoundaries();
            double urx = boundaries[0];
            double ury = boundaries[1];
            double llx = boundaries[2];
            double lly = boundaries[3];

            if (showAllTrees || tree.getName().equalsIgnoreCase(targetName)) {
                double[] coords = tree.getLoc().getCoords();
                double yval = (double) h - h * ((coords[1] - llx) / (urx - llx));
                double xval = (double) w - w * ((coords[0] - lly) / (ury - lly));

                if (yval < h & yval > 0 & xval < w & xval > 0) {
                    Circle circle = new Circle();
                    circle.setCenterX(xval);
                    circle.setCenterY(yval);
                    circle.setRadius(3);
                    circle.setFill(Color.RED);
                    treeView.getAnchorRoot().getChildren().add(circle); //attach each circle to the scene graph
                    undoStack.add(circle); //add the circle to the undo stack
                }
            }
        }

        txtSummary.setText("Trees selected: " + undoStack.size());
    }
}
