import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TextField;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.layout.Priority;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.stage.Stage;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.*;

import javafx.scene.AccessibleRole; //for screen reader support
import javafx.scene.AccessibleAction; //for screen reader support
import treetools.MunicipalTree;
import treetools.TreeInfo;
import treetools.TreeReader;

/**
 * This is the main program.
 * 
 * You may change this class if you wish, but you don't have to.
 */
public class TreeViewer extends Application {

    private Stack undoStack;
    private List<MunicipalTree> trees;
    private AnchorPane anchorRoot;

    TextField txtSummary;
    ChoiceBox treeSelect;


    private final int h = 700; //dimensions of the map for display
    private final int w = 1000;

    //UBC COORDINATES
//    private final double [] boundaries = {49.277419, -123.225205, 49.252539, -123.170316}; //map boundaries
//    String mName = "map-ubc.png";
//    String fName = "treelist-ubc.csv";
//    String location = "Vancouver";

    //    UTM COORDINATES -- UNCOMMENT FOR MISSISSAUGA MAP/TREES
    private final double [] boundaries = {43.55618, -79.68072, 43.52961, -79.62886}; //map boundaries
    String mName = "map-utm.png";
    String fName = "treelist-utm.csv";
    String location = "Mississauga";

    public static void main(String[] args) {
        launch(args);
    }

    public void start(Stage primaryStage) {

        //get the tree list from the file
        try {
            String filename = "assets/" + fName; //change this accordingly
            trees = TreeReader.readTreesFromFile(filename);
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }

        //load the map to visualize
        FileInputStream input = null;
        try {
            input = new FileInputStream("assets/" + mName); //change this accordingly
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

        Image image = new Image(input, w, h, false, false);
        ImageView iv = new ImageView(image);
        iv.setFitHeight(h);
        iv.setFitWidth(w);
        iv.setAccessibleRole(AccessibleRole.IMAGE_VIEW);
        iv.setAccessibleText("This is a map of" + location + "."); //This will be explanatory text for a screen reader
        iv.setAccessibleHelp("This is a map of " + location + " with dots at its tree locations."); //This will be help text for a screen reader

        anchorRoot = new AnchorPane();
        anchorRoot.getChildren().add(iv);

        double[] coords;
        undoStack = new Stack<>();
        for (MunicipalTree t: trees) {
            coords = t.getLoc().getCoords();
            double yval = (double) h - h*((coords[1] - boundaries[2])/(boundaries[0] - boundaries[2]));
            double xval = (double) w - w*((coords[0] - boundaries[3])/(boundaries[1] - boundaries[3]));
            if (yval < h & yval > 0 & xval < w & xval > 0) {
                Circle circle = new Circle();
                circle.setCenterX(xval);
                circle.setCenterY(yval);
                circle.setRadius(3);
                circle.setFill(Color.RED);
                anchorRoot.getChildren().add(circle);
                undoStack.add(circle);
            }
        }

        final Set<String> types = TreeInfo.getTreeTypes(trees);
        List<String> sortedList = new ArrayList<>(types);
        Collections.sort(sortedList);
        sortedList.add(0, "ALL TREES");

        treeSelect = new ChoiceBox(FXCollections.observableArrayList(sortedList));
        treeSelect.setId("treeSelect");
        treeSelect.setAccessibleRole(AccessibleRole.COMBO_BOX);
        treeSelect.setAccessibleText("This is a combo box for tree selection."); //This will be explanatory text for a screen reader
        treeSelect.setAccessibleHelp("This is a combo box to be used to make a tree selection. You can list options with control+option+space, and move up and down with arrows. To select, hit control+option+space."); //This will be help text for a screen reader

        txtSummary = new TextField();
        txtSummary.setId("txtSummary");
        txtSummary.setText("Trees selected: " + undoStack.size());
        txtSummary.setEditable(false);
        txtSummary.setAccessibleRole(AccessibleRole.TEXT_FIELD);
        txtSummary.setAccessibleText("Trees selected: " + undoStack.size()); //This will be explanatory text for a screen reader
        txtSummary.setAccessibleHelp("This is a summary of the trees of a given type."); //This will be help text for a screen reader

        TreeFilterEventHandler treeHandler = new TreeFilterEventHandler(treeSelect, txtSummary, this);

        Button filterTrees = new Button("Filter Tree List"){
                //Accessible actions can be triggered by screen readers or other
                //assistive technologies. Here we are saying that this button
                //object will be listening for AccessibleActions.
                //Note that the dots in the method signature that follows
                //means that zero or more Objects (or an array of them) may be passed
                //as the argument(s) to the executeAccessibleAction method.
                //FIRE is a type of AccessibleAction; for a list of all AccessibleActions
                //see https://openjfx.io/javadoc/18/javafx.graphics/javafx/scene/AccessibleAction.html
                public void executeAccessibleAction (AccessibleAction action, Object...parameters){
                switch (action) {
                    case FIRE:
                        String treeType = (String) treeSelect.getValue();
                        treeHandler.filterTrees(treeType);
                        txtSummary.setText("Trees selected: " + undoStack.size());
                        txtSummary.setAccessibleText("Trees selected: " + undoStack.size()); //This will be explanatory text for a screen reader
                    default:
                        super.executeAccessibleAction(action);
                }
            }
        };

        filterTrees.setId("filterTrees");
        filterTrees.setAccessibleRole(AccessibleRole.BUTTON); //each ball is a BUTTON
        filterTrees.setAccessibleText("This is a button that will allow you to filter trees."); //This will be explanatory text for a screen reader
        filterTrees.setAccessibleHelp("This is a button; click to filter the trees on display."); //This will be help text for a screen reader

        filterTrees.addEventHandler(MouseEvent.MOUSE_CLICKED, treeHandler);

        HBox hbox = new HBox(treeSelect, filterTrees, txtSummary);
        HBox.setHgrow(txtSummary, Priority.ALWAYS);
        hbox.setPadding(new Insets(10));
        hbox.setSpacing(10);

        VBox vbox = new VBox(hbox, anchorRoot);

        Scene scene = new Scene(vbox);
        primaryStage.setScene(scene);
        primaryStage.show();

    }

    public double[] getBoundaries() { return this.boundaries; }

    public int getHeight() {
        return h;
    }

    public int getWidth() {
        return w;
    }

    public AnchorPane getAnchorRoot() {
        return anchorRoot;
    }

    public Stack getUndoStack() {
        return undoStack;
    }

    public List<MunicipalTree> getTrees() { return trees; }

    public ChoiceBox getTreeSelect() {
        return this.treeSelect;
    }

    public TextField getTxtSummary() {
        return this.txtSummary;
    }


}
