import org.junit.jupiter.api.Test;
import java.lang.reflect.Field;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import guidelines.Forester;
import guidelines.Guidelines;
import guidelines.observer.GuidelineObserver;


import static org.junit.jupiter.api.Assertions.*;


class ObserverSolution implements GuidelineObserver {
    public Boolean notified = false;
    private HashMap<String, ArrayList<String>> treeGuidelines = new HashMap<String, ArrayList<String>>();
    public void update(HashMap<String, ArrayList<String>> observerState) {
        this.treeGuidelines = observerState;
        notified = true;
    }
    public List<String> getGuidelines(String treeName) {
        return this.treeGuidelines.get(treeName);
    }
}

public class HiddenTests {

    @Test
    void registerHiddenTest() throws IllegalAccessException {
        Guidelines guidelines = new Guidelines();
        GuidelineObserver o1 = new ObserverSolution();
        GuidelineObserver o2 = new ObserverSolution();

        guidelines.register(o1);
        //guidelines.register(o1);
        guidelines.register(o2);
        guidelines.register(o2);

        try {
            Field field = guidelines.getClass().getDeclaredField("foresterList");
            field.setAccessible(true);
            @SuppressWarnings("unchecked")
            List<GuidelineObserver> foresterList = (ArrayList<GuidelineObserver>) field.get(guidelines);
            assertTrue(foresterList.contains(o1), "Missing observer in foresterList.");
            assertTrue(foresterList.contains(o2), "Missing observer in foresterList.");
            assertEquals(2, foresterList.toArray().length, "Once an observer has subscribed, no point in subscribing them again!  You're resubscribing observers that are already observing.");
        } catch (NoSuchFieldException e) {
            fail("Can't find foresterList. Have you modified the given variable?");
        }
    }

    @Test
    void unregisterHiddenTest() throws IllegalAccessException {
        Guidelines forestryGuidelines = new Guidelines();
        GuidelineObserver o1 = new ObserverSolution();
        GuidelineObserver o2 = new ObserverSolution();

        forestryGuidelines.register(o1);
        //forestryGuidelines.register(o1);
        forestryGuidelines.register(o2);
        //forestryGuidelines.register(o2);

        forestryGuidelines.unregister(o1);
        //guidelines.unregister(o2);

        try {
            Field field = forestryGuidelines.getClass().getDeclaredField("foresterList");
            field.setAccessible(true);
            @SuppressWarnings("unchecked")
            List<GuidelineObserver> foresterList = (ArrayList<GuidelineObserver>) field.get(forestryGuidelines);

            assertFalse(foresterList.contains(o1), "Removed observer still in foresterList.");
            assertTrue(foresterList.contains(o2), "Missing observer in foresterList.");
            assertEquals(1, foresterList.toArray().length);
        } catch (NoSuchFieldException e) {
            fail("Can't find foresterList. Have you changed the given variable?");
        }
    }

    @Test
    void notifyObserversHiddenTest () {
        Guidelines forestryGuidelines = new Guidelines();
        ObserverSolution o1 = new ObserverSolution();
        ObserverSolution o2 = new ObserverSolution();

        forestryGuidelines.register(o1);
        forestryGuidelines.register(o2);

        assertFalse(o1.notified, "Observers shouldn't be notified before calling Guidelines.notify().");
        assertFalse(o2.notified, "Observers shouldn't be notified before calling Guidelines.notify().");

        forestryGuidelines.notifyObservers();

        assertTrue(o1.notified, "Observer was not notified.");
        assertTrue(o2.notified, "Observer was not notified.");
    }

    @Test
    void addGuidelineHiddenTest() {
        PrintStream stdOut = System.out;
        System.setOut(new PrintStream(new OutputStream() {
            public void write(int b) {}
        }));

        Guidelines forestryGuidelines = new Guidelines();
        assertEquals(Collections.emptyMap(), forestryGuidelines.getGuidelines()); // empty

        forestryGuidelines.addGuideline("name1", "env1"); // name1: [env1]
        forestryGuidelines.addGuideline("name2", "env1"); // name1: [env1], name2: [env1]

        assertEquals(List.of("env1"), forestryGuidelines.getGuidelines().get("name1"),
                "Guideline was not correctly added.");
        assertEquals(List.of("env1"), forestryGuidelines.getGuidelines().get("name2"),
                "Guideline was not correctly added.");

        forestryGuidelines.addGuideline("name1", "env2"); // name1: [env1, env2], name2: [env1]
        assertEquals(List.of("env1", "env2"), forestryGuidelines.getGuidelines().get("name1"),
                "Newly added guideline was not properly added.");

        forestryGuidelines.addGuideline("name1", "env2"); // shouldn't change anything
        //assertEquals(List.of("env1", "env2"), forestryGuidelines.getGuidelines().get("name1"),
        //        "");
        System.setOut(stdOut);
    }

    @Test
    void removeGuidelineHiddenTest() {
        Guidelines forestryGuidelines = new Guidelines();
        try {
            forestryGuidelines.removeGuideline("name", "env"); // shouldn't change anything
        } catch (Exception e) {
            fail("Removing guideline from empty Guidelines class shouldn't raise exception.");
        }

        forestryGuidelines.addGuideline("name1", "env1");
        forestryGuidelines.addGuideline("name1", "env2");
        forestryGuidelines.addGuideline("name2", "env1"); // name1: [env1, env2], name2: [env1]

        forestryGuidelines.removeGuideline("name1", "env1");
        assertEquals(List.of("env2"), forestryGuidelines.getGuidelines().get("name1"),
                "Guideline was not properly removed.");
        assertEquals(List.of("env1"), forestryGuidelines.getGuidelines().get("name2"),
                "Guideline was not properly removed.");

        forestryGuidelines.removeGuideline("name1", "env2");
        assertEquals(Collections.emptyList(), forestryGuidelines.getGuidelines().get("name1"),
                "Guideline was not properly removed.");

        forestryGuidelines.removeGuideline("name2", "env1");
        assertEquals(Collections.emptyList(), forestryGuidelines.getGuidelines().get("name2"),
                "Guideline was not properly removed.");
    }

    @Test
    void updateHiddenTest() {
        PrintStream stdOut = System.out;
        System.setOut(new PrintStream(new OutputStream() {
            public void write(int b) {}
        }));

        HashMap<String, ArrayList<String>> hashmap = new HashMap<String, ArrayList<String>>();
        hashmap.put("name", new ArrayList<>(List.of("env")));

        GuidelineObserver o = new Forester();
        o.update(hashmap);
        assertEquals(List.of("env"), ((Forester) o).getGuidelines("name"));

        System.setOut(stdOut);
    }
}
